#################################################################################
# Searching for G: A Critical Evaluation of SPM-LS dimensionality               #
#                                                                               #
# Journal of Intelligence, 2019                                                 #                                       
#                                                                               #
# Authors:                                                                      #
#   1. Eduardo Garcia-Garzon                                                    #
#   1. Francisco J. Abad                                                        #  
#   2. Luis E. Garrido                                                          #
#                                                                               #  
# Affiliations:                                                                 #
#   1. Facultad de Psicologia, Universidad Autonoma de Madrid                   #    
#   2. Facultad de Psicologia, Pontificia Universidad Católica Madre y Maestra  #
#                                                                               #
#   Last updated: 20-06-2019                                                    #                               
#################################################################################


# This script contains some functions required to compile Main results.R, Appendix A.R & Appendix B.R

## (Non-iterative)Empirical Target definition for two factors.
### Function based on Garcia-Garzon, Garrido & Abad (2019) method (https://doi.org/10.1027/1614-2241/a000163).


define_target <- function (L = NULL){
  
  ###################################
  # Input: L = A two-factor solution to be rotated towards a BEFA model.
  #
  # Output: Targ = Partially specified trget matrix.
  #                NA will be used for untargeted factor loadings and zero targets for factor loadings to be minimized.
  
  Targ = L
  c2 <- c()
  c2 <- Targ[,-1]/sqrt(sum(Targ[, -1]^2))
  c2 <- c2^2
  cuts_mix <- c()
  
  I <- sort(c2)
  j <- 1
  a <- c()
  
  for (j in 1:length(I)) {
    a[j] <- I[j] - I[j - 1]
  }
  
  Targ[which(I > mean(a, na.rm=T)),2] <- NA
  Targ[!is.na(Targ[,2]),2] <- 0
  Targ[,1] <- NA
  
  return (Targ)
  
}

## Explained common variance estimation

ecv <- function (L) {
  
  ###################################
  # Input: L = A factor solution from which ECV must be computed. 
  #            Factor loadings in the first column are expected to represent the general factor
  #
  # Output: EC = Explained common variance.
  
  ec <- sum(L[,1]^2) / (sum(L[,1]^2) + sum(L[,2:ncol(L)]^2))
  return (ec)
}



